#!/usr/bin/env python3

'''Parse protobuf and produce output readable by unparse.py.'''

import io
import sys

def read_bytes(f, n):
    bs = f.read(n)
    if len(bs) != n:
        raise RuntimeError('unexpected end of file')
    return bs

def read_byte(f):
    return read_bytes(f, 1)[0]

def parse_varint(f):
    varint = 0
    i = 0
    while True:
        b = read_byte(f)
        varint |= (b & 0x7f) << (i * 7)
        if not b & 0x80:
            break
        i += 1
    return varint

def parse_length_delimited(f):
    length = parse_varint(f)
    buf = read_bytes(f, length)
    buf_f = io.BytesIO(buf)
    v = buf
    if buf:
        try:
            v = parse_file(buf_f)
        except RuntimeError:
            pass
    return v

def parse_fixed32(f):
    buf = read_bytes(f, 4)
    return int.from_bytes(buf, 'little')

def parse_fixed64(f):
    buf = read_bytes(f, 8)
    return int.from_bytes(buf, 'little')

def parse_group(f):
    fields = []
    while True:
        field = parse_field(f)
        if field is None:
            raise RuntimeError('nonterminated group')
        field_number, wire_type, v = field
        if wire_type == 4:
            break
        fields.append(field)
    return fields

def parse_field(f):
    bs = f.read(1)
    if not bs:
        return None
    b = bs[0]
    field_number = (b >> 3) & 0x1f
    if field_number & 0x10:
        field_number = (field_number & 0xf) | (read_byte(f) << 4)
    if not 0 < field_number < 2048:
        raise RuntimeError('field number %d not > 0 and < 2048')
    wire_type = b & 0x7
    if wire_type == 0:
        v = parse_varint(f)
    elif wire_type == 1:
        v = parse_fixed64(f)
    elif wire_type == 2:
        v = parse_length_delimited(f)
    elif wire_type == 3:
        v = parse_group(f)
    elif wire_type == 4:
        v = None
    elif wire_type == 5:
        v = parse_fixed32(f)
    else:
        raise RuntimeError('unhandled wire type %d' % wire_type)
    return field_number, wire_type, v

def parse_file(f):
    fields = []
    while True:
        field = parse_field(f)
        if field is None:
            break
        field_number, wire_type, v = field
        if wire_type == 4:
            raise RuntimeError('not in group')
        fields.append(field)
    return fields

def main():
    print(repr(parse_file(sys.stdin.buffer)))

if __name__ == '__main__':
    main()
