#!/usr/bin/env python3

'''Unparse protobuf output produced by parse.py.'''

import io
import sys

DATA = [(1, 0, 1), (2, 2, b'\xe6\xa3\x80\xe6\x9f\xa5\xe6\x9b\xb4\xe6\x96\xb0'), (3, 0, 0), (4, 2, b'http://puds.ucweb.com/hello.exe'), (5, 2, b''), (7, 2, b''), (8, 2, b'5.7.11073.7'), (9, 2, b''), (10, 2, b''), (11, 0, 54320800), (12, 0, 0), (13, 2, b''), (14, 2, b''), (15, 2, b'\xe6\x9b\xb4\xe6\x96\xb0'), (16, 2, b'\xe5\x8f\x96\xe6\xb6\x88'), (17, 2, b''), (19, 0, 0), (20, 0, 0), (21, 2, b'https://puds.ucweb.com/hello.exe'), (22, 0, 0), (23, 2, b'ed1530f8b56468d1eab3654904b9aeae'), (24, 2, [(1, 2, b'hide_redpoint'), (2, 2, b'false')]), (24, 2, [(1, 2, b'apkstore_pkname'), (2, 2, b'')])]

def write_bytes(f, bs):
    f.write(bs)

def write_byte(f, b):
    f.write(bytes([b]))

def unparse_varint(f, varint):
    if varint < 0:
        raise ValueError('varint must be >= 0')
    while True:
        b = varint & 0x7f
        varint >>= 7
        if varint > 0:
            b |= 0x80
        write_byte(f, b)
        if varint == 0:
            break

def unparse_length_delimited(f, v):
    if isinstance(v, list):
        buf_f = io.BytesIO()
        unparse_file(buf_f, v)
        v = buf_f.getvalue()
    length = len(v)
    unparse_varint(f, length)
    write_bytes(f, v)

def unparse_fixed32(f, v):
    if not 0 <= v <= 0xffffffff:
        raise ValueError('fixed32 must be i : 0 <= i <= 0xffffffff')
    write_bytes(f, int.to_bytes(4, 'little'))

def unparse_fixed64(f, v):
    if not 0 <= v <= 0xffffffffffffffff:
        raise ValueError('fixed64 must be i : 0 <= i <= 0xffffffffffffffff')
    write_bytes(f, int.to_bytes(8, 'little'))

def unparse_field(f, field):
    field_number, wire_type, v = field
    if not 0 < field_number < 2048:
        raise ValueError('field number must be i : 0 < i < 2048')
    if not 0 <= wire_type <= 5:
        raise ValueError('wire type must be i : 0 <= i <= 5')
    b = wire_type | ((field_number & 0xf) << 3)
    field_number >>= 4
    if field_number > 0:
        b |= 0x80
    write_byte(f, b)
    if field_number > 0:
        write_byte(f, field_number)
    if wire_type == 0:
        unparse_varint(f, v)
    elif wire_type == 1:
        unparse_fixed64(f, v)
    elif wire_type == 2:
        unparse_length_delimited(f, v)
    elif wire_type == 5:
        unparse_fixed32(f, v)
    else:
        raise ValueError('unhandled wire type %d' % wire_type)
    return field_number, wire_type, v

def unparse_file(f, fields):
    for field in fields:
        unparse_field(f, field)

def main():
    unparse_file(sys.stdout.buffer, DATA)

if __name__ == '__main__':
    main()
